# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda application package

Data Exchange Standard for Archival
"""

from cubicweb_compound import skip_rtypes_set, structure_def, CompositeGraph


# control of compound graph by adding etype / rtype to the corresponding set below
GRAPH_SKIP_ETYPES = set()
GRAPH_SKIP_RTYPES = set(['container', 'clone_of'])


def seda_profile_container_def(schema):
    """Define container for SEDAArchiveTransfer, as a list of (etype, parent_rdefs)."""
    return structure_def(schema, 'SEDAArchiveTransfer',
                         skipetypes=GRAPH_SKIP_ETYPES, skiprtypes=GRAPH_SKIP_RTYPES).items()


def _iter_external_rdefs(eschema, skip_rtypes):
    """Return an iterator on (rdef, role) of external relations from entity schema (i.e.
    non-composite relations).
    """
    for rschema, targets, role in eschema.relation_definitions():
        if rschema in skip_rtypes:
            continue
        for target_etype in targets:
            rdef = eschema.rdef(rschema, role, target_etype)
            if rdef.composite:
                continue
            yield rdef, role


def iter_all_rdefs(schema, container_etype):
    """Return an iterator on (rdef, role) of all relations of the compound graph starting from the
    given entity type, both internal (composite) and external (non-composite).
    """
    graph = CompositeGraph(schema, skipetypes=GRAPH_SKIP_ETYPES, skiprtypes=GRAPH_SKIP_RTYPES)
    skip_external_rtypes = skip_rtypes_set(GRAPH_SKIP_RTYPES)
    stack = [container_etype]
    visited = set(stack)
    while stack:
        etype = stack.pop()
        for (rtype, role), targets in graph.child_relations(etype):
            rschema = schema.rschema(rtype)
            for target in targets:
                if role == 'subject':
                    rdef = rschema.rdefs[(etype, target)]
                else:
                    rdef = rschema.rdefs[(target, etype)]
                yield rdef, role

                if target not in visited:
                    visited.add(target)
                    stack.append(target)
        for rdef, role in _iter_external_rdefs(schema[etype], skip_external_rtypes):
            yield rdef, role



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from yams import BASE_TYPES

from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape

from cubicweb import tags, neg_role
from cubicweb.utils import UStringIO
from cubicweb.web import formfields as ff
# XXX alias to avoid weird side effect: uicfg will become our uicfg submodule
from cubicweb.web.views import primary, uicfg as cwuicfg

from ..xsd import XSDM_MAPPING
from ..xsd2uicfg import FIRST_LEVEL_ETYPES
from .uicfg import ETYPE_ATTR_DOC
from .widgets import SEDAMetaField


CONTENT_ETYPE = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'


pvs = cwuicfg.primaryview_section
afs = cwuicfg.autoform_section
aff = cwuicfg.autoform_field
affk = cwuicfg.autoform_field_kwargs
rec = cwuicfg.reledit_ctrl

pvs.tag_subject_of(('*', 'container', '*'), 'hidden')
pvs.tag_object_of(('*', 'container', '*'), 'hidden')
afs.tag_subject_of(('*', 'container', '*'), 'main', 'hidden')
afs.tag_object_of(('*', 'container', '*'), 'main', 'hidden')

pvs.tag_subject_of(('*', 'scheme_relation_type', '*'), 'hidden')
afs.tag_subject_of(('*', 'scheme_relation_type', '*'), 'main', 'hidden')
pvs.tag_subject_of(('*', 'scheme_entity_type', '*'), 'hidden')
afs.tag_subject_of(('*', 'scheme_entity_type', '*'), 'main', 'hidden')

pvs.tag_object_of(('*', 'seda_keyword_reference_to_scheme', '*'), 'hidden')
afs.tag_object_of(('*', 'seda_keyword_reference_to_scheme', '*'), 'main', 'hidden')

afs.tag_subject_of(('*', 'code_keyword_type', '*'), 'main', 'attributes')
pvs.tag_subject_of(('*', 'code_keyword_type', '*'), 'attributes')
afs.tag_object_of(('*', 'code_keyword_type', '*'), 'main', 'hidden')
pvs.tag_object_of(('*', 'code_keyword_type', '*'), 'hidden')

aff.tag_attribute(('*', 'user_cardinality'), SEDAMetaField)
afs.tag_attribute(('*', 'user_annotation'), 'main', 'hidden')
for etype in FIRST_LEVEL_ETYPES:
    aff.tag_attribute((etype, 'user_cardinality'), ff.StringField)
    afs.tag_attribute((etype, 'user_annotation'), 'main', 'attributes')

pvs.tag_attribute(('*', 'ordering'), 'hidden')
afs.tag_attribute(('*', 'ordering'), 'main', 'hidden')


def rtags_from_xsd_element(etype, element_name):
    """Return primary view section and display control rtags, generated from information in the XSD
    for the given element name.
    """
    rtype_role_targets = ((rtype, role, path[-1][-2])
                          for rtype, role, path in XSDM_MAPPING.iter_rtype_role(element_name))
    return rtags_from_rtype_role_targets(etype, rtype_role_targets)


def rtags_from_rtype_role_targets(etype, rtype_role_targets):
    """Return primary view section and display control rtags from a list of (`rtype`, `role`,
    `target entity type`).
    """
    ordered = []
    rsection = cwuicfg.PrimaryViewSectionRelationTags(__module__=__name__)
    for rtype, role, target in rtype_role_targets:
        if role == 'subject':
            if target in BASE_TYPES:
                # attribute handled by the main tab (e.g. filename), skip it
                continue
            # mandatory elements
            relation = (etype, rtype, '*', role)
            rec.tag_subject_of(('*', rtype, '*'),
                               {'rvid': 'seda.reledit.simplelink'})
        else:
            relation = ('*', rtype, etype, role)
            if target in BASE_TYPES or not rtype.endswith('_from'):
                vid = 'seda.reledit.text'
            else:
                vid = 'seda.reledit.complexlink'
            rec.tag_object_of(('*', rtype, '*'),
                              {'rvid': vid,
                               'novalue_label': ' '})
        rsection.tag_relation(relation, 'attributes')
        ordered.append((rtype, role))
    display_ctrl = cwuicfg.DisplayCtrlRelationTags(__module__=__name__)
    display_ctrl.set_fields_order(etype, ordered)
    return rsection, display_ctrl


def add_subobject_link(entity, rtype, role, extraurlparams, msg=None, klass=None):
    """Return a HTML link to add a subobject linked to `entity` through `rtype` relation where
    entity has `role`.

    If the user doesn't have the permission to add the relation or subobject, return an empty
    string.
    """
    req = entity._cw
    rschema = req.vreg.schema[rtype]
    targets = rschema.targets(role=role)
    assert len(targets) == 1, 'expecting a single {0} for relation {1}'.format(role, rschema)
    target_etype = targets[0].type
    if not has_rel_perm('add', entity, rtype, role, target_etype=target_etype):
        return u''
    linkto = '{rtype}:{eid}:{role}'.format(rtype=rtype, eid=entity.eid, role=neg_role(role))
    urlparams = {'__linkto': linkto,
                 '__redirectpath': entity.rest_path()}
    urlparams.update(extraurlparams)
    if msg is None:
        msg = req.__(rtype if role == 'subject' else rtype + '_object')
    return add_etype_link(req, target_etype, urlparams, msg=msg, klass=klass)


def add_etype_link(req, etype, urlparams, msg=u'', klass=None):
    """Return an HTML link to add an entity of type `etype`, or an empty string if the user doesn't
    have the permission.
    """
    vreg = req.vreg
    eschema = vreg.schema.eschema(etype)
    if eschema.has_perm(req, 'add'):
        if klass is None:
            klass = 'icon-plus-circled pull-right'
        url = vreg['etypes'].etype_class(etype).cw_create_url(req, **urlparams)
        return tags.a(msg, href=url, klass=klass)
    return u''


def has_rel_perm(action, entity, rtype, role, target_etype=None, target_entity=None):
    """Return True if the user has the permission for `action` on the relation `rtype` where
    `entity` is `role`. Either target entity type or target entity could also be specified.
    """
    if role == 'subject':
        kwargs = {'fromeid': entity.eid}
        if target_entity is not None:
            kwargs['toeid'] = target_entity.eid
    else:
        kwargs = {'toeid': entity.eid}
        if target_entity is not None:
            kwargs['fromeid'] = target_entity.eid
    if target_entity is not None:
        assert not target_etype
        target_etype = target_entity.cw_etype
    rdef = entity.e_schema.rdef(rtype, role, target_etype)
    return rdef.has_perm(entity._cw, action, **kwargs)


def dropdown_button(text, links):
    """Return an HTML button with `text` and dropdown content from `links`.
    """
    data = UStringIO()
    w = data.write
    w(u'<div class="btn-group pull-right clearfix">')
    w(tags.button(
        u' '.join([text, tags.span(klass='caret')]),
        escapecontent=False,
        klass='btn btn-success dropdown-toggle',
        **{'data-toggle': 'dropdown', 'aria-expanded': 'false'}))
    w(u'<ul class="dropdown-menu" role="menu">')
    for link in links:
        w(u'<li>{0}</li>'.format(link))
    w(u'</ul>')
    w(u'</div>')
    return data.getvalue()


orig_rel_label = primary.PrimaryView._rel_label


@monkeypatch(primary.PrimaryView)
def _rel_label(self, entity, rschema, role, dispctrl):
    label = orig_rel_label(self, entity, rschema, role, dispctrl)
    try:
        element_name, desc = ETYPE_ATTR_DOC[(entity.cw_etype, rschema.type, role)]
    except KeyError:
        return label
    description = xml_escape(desc[0]) if desc else u''
    title = xml_escape(element_name)
    self._cw.add_onload("$('.popOverLabel').popover();")
    return (u'<div class="popOverLabel" '
            'data-toggle="popover" data-placement="top" '
            'title="{}" data-content="{}">{}</div>'.format(
                title, description, label))



# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda views for ArchiveUnit"""

from cubicweb import _
from cubicweb.predicates import adaptable, is_instance
from cubicweb.web import component

from ..entities import simplified_profile, component_unit
from ..entities.itree import move_child_at_index, reparent
from . import jqtree


class BaseTreeComponent(component.EntityCtxComponent):
    """Display the whole profile tree."""
    __regid__ = 'seda.tree'
    __select__ = (component.EntityCtxComponent.__select__
                  & adaptable('ITreeBase'))
    __abstract__ = True
    context = 'left'
    order = -1

    def render_body(self, w):
        self._cw.add_css('cubes.jqtree.css')
        self._cw.add_js('cubes.seda.js')
        self.entity.view('jqtree.treeview', w=w)


class ProfileTreeComponent(BaseTreeComponent):
    __select__ = (
        BaseTreeComponent.__select__
        & is_instance(
            'SEDAArchiveTransfer',
            'SEDAArchiveUnit',
            'SEDABinaryDataObject',
            'SEDAPhysicalDataObject',
        )
    )
    title = _('SEDA profile tree')


class UnitTreeComponent(BaseTreeComponent):
    __select__ = BaseTreeComponent.__select__ & component_unit()
    title = _('Archive unit component tree')


class SEDAIJQTreeAdapter(jqtree.IJQTreeAdapter):
    js_can_move_to = 'seda.canMoveTo'


class ArchiveTransferIJQTreeAdapter(SEDAIJQTreeAdapter):
    __select__ = (SEDAIJQTreeAdapter.__select__
                  & is_instance('SEDAArchiveTransfer'))

    def maybe_parent_of(self):
        return ['SEDAArchiveUnit',
                'SEDAPhysicalDataObject', 'SEDABinaryDataObject']

    def move_child_at_index(self, ceid, index, reparenting=False):
        move_child_at_index(self._cw, self.entity.eid, ceid, index, reparenting)


class SimplifiedArchiveTransferIJQTreeAdapter(ArchiveTransferIJQTreeAdapter):
    __select__ = ArchiveTransferIJQTreeAdapter.__select__ & simplified_profile()

    def maybe_parent_of(self):
        return ['SEDAArchiveUnit']


class ArchiveUnitIJQTreeAdapter(SEDAIJQTreeAdapter):
    __select__ = (SEDAIJQTreeAdapter.__select__
                  & is_instance('SEDAArchiveUnit'))

    def maybe_parent_of(self):
        return [] if self.entity.is_archive_unit_ref else ['SEDAArchiveUnit']

    def maybe_moved(self):
        return True

    def reparent(self, peid, index):
        parent = self._cw.entity_from_eid(peid)
        if parent.cw_etype == 'SEDAArchiveUnit':
            parent = parent.first_level_choice.content_sequence
        else:
            assert parent.cw_etype == 'SEDAArchiveTransfer', (
                'cannot re-parent to entity type {0}'.format(parent.cw_etype))
        reparent(self.entity, parent.eid, index)

    def move_child_at_index(self, ceid, index, reparenting=False):
        move_child_at_index(self._cw,
                            self.entity.first_level_choice.content_sequence.eid,
                            ceid, index, reparenting)


class SimplifiedArchiveUnitIJQTreeAdapter(ArchiveUnitIJQTreeAdapter):
    __select__ = ArchiveUnitIJQTreeAdapter.__select__ & simplified_profile()

    def maybe_parent_of(self):
        return [] if self.entity.is_archive_unit_ref else ['SEDAArchiveUnit',
                                                           'SEDABinaryDataObject']

    def move_child_at_index(self, ceid, index, reparenting=False):
        # when reordering a data object, we actually have to maintain ordering
        # of its reference
        child = self._cw.entity_from_eid(ceid)
        if child.cw_etype == 'SEDABinaryDataObject':
            child = child.reverse_seda_data_object_reference_id[0]
        move_child_at_index(self._cw,
                            self.entity.first_level_choice.content_sequence.eid,
                            child.eid, index, reparenting)


class DataObjectIJQTreeAdapter(SEDAIJQTreeAdapter):
    __select__ = (SEDAIJQTreeAdapter.__select__
                  & is_instance('SEDABinaryDataObject', 'SEDAPhysicalDataObject'))

    def maybe_moved(self):
        return True


class SimplifiedBinaryDataObjectIJQTreeAdapter(SEDAIJQTreeAdapter):
    __select__ = (SEDAIJQTreeAdapter.__select__
                  & is_instance('SEDABinaryDataObject') & simplified_profile())
    rtype_to_archivetransfer = 'seda_binary_data_object'

    def maybe_moved(self):
        return True

    def reparent(self, peid, index):
        archunit = self._cw.entity_from_eid(peid)
        parent = archunit.first_level_choice.content_sequence
        child = self.entity.reverse_seda_data_object_reference_id[0]
        reparent(child, parent.eid, index)



# XXX unsupported merge because of incompatible cardinality SEDADataObjectVersion seda_data_object_version_to Concept ? <SEDADataObjectVersion seda_data_object_version_to set(['Concept'])> ?*
# XXX unsupported merge because of incompatible cardinality SEDAArchiveUnitRefId seda_archive_unit_ref_id_to SEDAArchiveUnit ? <SEDAArchiveUnitRefId seda_archive_unit_ref_id_to set(['SEDAArchiveUnit'])> ?*
# XXX unsupported merge because of incompatible cardinality SEDAArchiveUnitRefId seda_archive_unit_ref_id_to SEDAArchiveUnit ? <SEDAArchiveUnitRefId seda_archive_unit_ref_id_to set(['SEDAArchiveUnit'])> ?*
# XXX unsupported merge because of incompatible cardinality SEDAArchiveUnitRefId seda_archive_unit_ref_id_to SEDAArchiveUnit ? <SEDAArchiveUnitRefId seda_archive_unit_ref_id_to set(['SEDAArchiveUnit'])> ?*
# XXX unsupported merge because of incompatible cardinality SEDAArchiveUnitRefId seda_archive_unit_ref_id_to SEDAArchiveUnit ? <SEDAArchiveUnitRefId seda_archive_unit_ref_id_to set(['SEDAArchiveUnit'])> ?*
# XXX unsupported merge because of incompatible cardinality SEDAArchiveUnitRefId seda_archive_unit_ref_id_to SEDAArchiveUnit ? <SEDAArchiveUnitRefId seda_archive_unit_ref_id_to set(['SEDAArchiveUnit'])> ?*
# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""THIS FILE IS GENERATED FROM SEDA 2.0 XSD FILES, DO NOT EDIT"""

from yams.buildobjs import EntityType, RelationDefinition
from yams.buildobjs import String, Boolean
from cubicweb.schema import RQLConstraint
from cubicweb_seda.schema import seda_profile_element


@seda_profile_element()
class SEDAArchiveTransfer(EntityType):
    u""""""


@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAComment(EntityType):
    u""""""
    comment = String(fulltextindexed=True)


class archive_transfer_comment(RelationDefinition):
    name = 'seda_comment'
    subject = 'SEDAComment'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASignature(EntityType):
    u""""""


class archive_transfer_signature(RelationDefinition):
    name = 'seda_signature'
    subject = 'SEDASignature'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAArchivalAgreement(EntityType):
    u""""""
    archival_agreement = String(fulltextindexed=True)


class archive_transfer_archival_agreement(RelationDefinition):
    name = 'seda_archival_agreement'
    subject = 'SEDAArchivalAgreement'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDARelatedTransferReference(EntityType):
    u""""""


class archive_transfer_related_transfer_reference(RelationDefinition):
    name = 'seda_related_transfer_reference'
    subject = 'SEDARelatedTransferReference'
    object = 'SEDAArchiveTransfer'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDATransferRequestReplyIdentifier(EntityType):
    u""""""


class archive_transfer_transfer_request_reply_identifier(RelationDefinition):
    name = 'seda_transfer_request_reply_identifier'
    subject = 'SEDATransferRequestReplyIdentifier'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class archive_transfer_archival_agency(RelationDefinition):
    name = 'seda_archival_agency'
    subject = 'SEDAArchiveTransfer'
    object = 'AuthorityRecord'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

class archive_transfer_transferring_agency(RelationDefinition):
    name = 'seda_transferring_agency'
    subject = 'SEDAArchiveTransfer'
    object = 'AuthorityRecord'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAMessageDigestAlgorithmCodeListVersion(EntityType):
    u""""""


class archive_transfer_message_digest_algorithm_code_list_version_from(RelationDefinition):
    name = 'seda_message_digest_algorithm_code_list_version_from'
    subject = 'SEDAMessageDigestAlgorithmCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class message_digest_algorithm_code_list_version_message_digest_algorithm_code_list_version_to(RelationDefinition):
    name = 'seda_message_digest_algorithm_code_list_version_to'
    subject = 'SEDAMessageDigestAlgorithmCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAMimeTypeCodeListVersion(EntityType):
    u""""""


class archive_transfer_mime_type_code_list_version_from(RelationDefinition):
    name = 'seda_mime_type_code_list_version_from'
    subject = 'SEDAMimeTypeCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class mime_type_code_list_version_mime_type_code_list_version_to(RelationDefinition):
    name = 'seda_mime_type_code_list_version_to'
    subject = 'SEDAMimeTypeCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEncodingCodeListVersion(EntityType):
    u""""""


class archive_transfer_encoding_code_list_version_from(RelationDefinition):
    name = 'seda_encoding_code_list_version_from'
    subject = 'SEDAEncodingCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class encoding_code_list_version_encoding_code_list_version_to(RelationDefinition):
    name = 'seda_encoding_code_list_version_to'
    subject = 'SEDAEncodingCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAFileFormatCodeListVersion(EntityType):
    u""""""


class archive_transfer_file_format_code_list_version_from(RelationDefinition):
    name = 'seda_file_format_code_list_version_from'
    subject = 'SEDAFileFormatCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class file_format_code_list_version_file_format_code_list_version_to(RelationDefinition):
    name = 'seda_file_format_code_list_version_to'
    subject = 'SEDAFileFormatCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACompressionAlgorithmCodeListVersion(EntityType):
    u""""""


class archive_transfer_compression_algorithm_code_list_version_from(RelationDefinition):
    name = 'seda_compression_algorithm_code_list_version_from'
    subject = 'SEDACompressionAlgorithmCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class compression_algorithm_code_list_version_compression_algorithm_code_list_version_to(RelationDefinition):
    name = 'seda_compression_algorithm_code_list_version_to'
    subject = 'SEDACompressionAlgorithmCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADataObjectVersionCodeListVersion(EntityType):
    u""""""


class archive_transfer_data_object_version_code_list_version_from(RelationDefinition):
    name = 'seda_data_object_version_code_list_version_from'
    subject = 'SEDADataObjectVersionCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class data_object_version_code_list_version_data_object_version_code_list_version_to(RelationDefinition):
    name = 'seda_data_object_version_code_list_version_to'
    subject = 'SEDADataObjectVersionCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAStorageRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_storage_rule_code_list_version_from(RelationDefinition):
    name = 'seda_storage_rule_code_list_version_from'
    subject = 'SEDAStorageRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class storage_rule_code_list_version_storage_rule_code_list_version_to(RelationDefinition):
    name = 'seda_storage_rule_code_list_version_to'
    subject = 'SEDAStorageRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAppraisalRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_appraisal_rule_code_list_version_from(RelationDefinition):
    name = 'seda_appraisal_rule_code_list_version_from'
    subject = 'SEDAAppraisalRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class appraisal_rule_code_list_version_appraisal_rule_code_list_version_to(RelationDefinition):
    name = 'seda_appraisal_rule_code_list_version_to'
    subject = 'SEDAAppraisalRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAccessRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_access_rule_code_list_version_from(RelationDefinition):
    name = 'seda_access_rule_code_list_version_from'
    subject = 'SEDAAccessRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class access_rule_code_list_version_access_rule_code_list_version_to(RelationDefinition):
    name = 'seda_access_rule_code_list_version_to'
    subject = 'SEDAAccessRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADisseminationRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_dissemination_rule_code_list_version_from(RelationDefinition):
    name = 'seda_dissemination_rule_code_list_version_from'
    subject = 'SEDADisseminationRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class dissemination_rule_code_list_version_dissemination_rule_code_list_version_to(RelationDefinition):
    name = 'seda_dissemination_rule_code_list_version_to'
    subject = 'SEDADisseminationRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAReuseRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_reuse_rule_code_list_version_from(RelationDefinition):
    name = 'seda_reuse_rule_code_list_version_from'
    subject = 'SEDAReuseRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class reuse_rule_code_list_version_reuse_rule_code_list_version_to(RelationDefinition):
    name = 'seda_reuse_rule_code_list_version_to'
    subject = 'SEDAReuseRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAClassificationRuleCodeListVersion(EntityType):
    u""""""


class archive_transfer_classification_rule_code_list_version_from(RelationDefinition):
    name = 'seda_classification_rule_code_list_version_from'
    subject = 'SEDAClassificationRuleCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class classification_rule_code_list_version_classification_rule_code_list_version_to(RelationDefinition):
    name = 'seda_classification_rule_code_list_version_to'
    subject = 'SEDAClassificationRuleCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAcquisitionInformationCodeListVersion(EntityType):
    u""""""


class archive_transfer_acquisition_information_code_list_version_from(RelationDefinition):
    name = 'seda_acquisition_information_code_list_version_from'
    subject = 'SEDAAcquisitionInformationCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class acquisition_information_code_list_version_acquisition_information_code_list_version_to(RelationDefinition):
    name = 'seda_acquisition_information_code_list_version_to'
    subject = 'SEDAAcquisitionInformationCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDARelationshipCodeListVersion(EntityType):
    u""""""


class archive_transfer_relationship_code_list_version_from(RelationDefinition):
    name = 'seda_relationship_code_list_version_from'
    subject = 'SEDARelationshipCodeListVersion'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class relationship_code_list_version_relationship_code_list_version_to(RelationDefinition):
    name = 'seda_relationship_code_list_version_to'
    subject = 'SEDARelationshipCodeListVersion'
    object = 'ConceptScheme'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDABinaryDataObject(EntityType):
    u""""""
    filename = String(fulltextindexed=True)


class archive_transfer_binary_data_object(RelationDefinition):
    name = 'seda_binary_data_object'
    subject = 'SEDABinaryDataObject'
    object = 'SEDAArchiveTransfer'
    cardinality = '?*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAPhysicalDataObject(EntityType):
    u""""""


class archive_transfer_physical_data_object(RelationDefinition):
    name = 'seda_physical_data_object'
    subject = 'SEDAPhysicalDataObject'
    object = 'SEDAArchiveTransfer'
    cardinality = '?*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDARelationship(EntityType):
    u""""""


class binary_data_object_relationship(RelationDefinition):
    name = 'seda_relationship'
    subject = 'SEDARelationship'
    object = 'SEDABinaryDataObject'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADataObjectVersion(EntityType):
    u""""""


class binary_data_object_data_object_version_from(RelationDefinition):
    name = 'seda_data_object_version_from'
    subject = 'SEDADataObjectVersion'
    object = ('SEDABinaryDataObject', 'SEDAPhysicalDataObject')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class data_object_version_data_object_version_to(RelationDefinition):
    name = 'seda_data_object_version_to'
    subject = 'SEDADataObjectVersion'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_data_object_version_code_list_version_from AT, CACLV seda_data_object_version_code_list_version_to CS,S container AT')]

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltBinaryDataObjectAttachment(EntityType):
    u""""""


class binary_data_object_alt_binary_data_object_attachment(RelationDefinition):
    name = 'seda_alt_binary_data_object_attachment'
    subject = 'SEDABinaryDataObject'
    object = 'SEDAAltBinaryDataObjectAttachment'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACompressed(EntityType):
    u""""""
    compressed = Boolean()


class binary_data_object_compressed(RelationDefinition):
    name = 'seda_compressed'
    subject = 'SEDACompressed'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class physical_data_object_relationship(RelationDefinition):
    name = 'seda_relationship'
    subject = 'SEDARelationship'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAArchiveUnit(EntityType):
    u""""""


class archive_transfer_archive_unit(RelationDefinition):
    name = 'seda_archive_unit'
    subject = 'SEDAArchiveUnit'
    object = 'SEDAArchiveTransfer'
    cardinality = '?*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAServiceLevel(EntityType):
    u""""""
    service_level = String(fulltextindexed=True)


class archive_transfer_service_level(RelationDefinition):
    name = 'seda_service_level'
    subject = 'SEDAServiceLevel'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAcquisitionInformation(EntityType):
    u""""""


class archive_transfer_acquisition_information_from(RelationDefinition):
    name = 'seda_acquisition_information_from'
    subject = 'SEDAAcquisitionInformation'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class acquisition_information_acquisition_information_to(RelationDefinition):
    name = 'seda_acquisition_information_to'
    subject = 'SEDAAcquisitionInformation'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACLV seda_acquisition_information_code_list_version_from AT,        CACLV seda_acquisition_information_code_list_version_to CS,       S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT,            RT name "seda_acquisition_information_to")')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDALegalStatus(EntityType):
    u""""""


class archive_transfer_legal_status_from(RelationDefinition):
    name = 'seda_legal_status_from'
    subject = 'SEDALegalStatus'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class legal_status_legal_status_to(RelationDefinition):
    name = 'seda_legal_status_to'
    subject = 'SEDALegalStatus'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_legal_status_to"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAOriginatingAgencyIdentifier(EntityType):
    u""""""
    originating_agency_identifier = String(fulltextindexed=True)


class archive_transfer_originating_agency_identifier(RelationDefinition):
    name = 'seda_originating_agency_identifier'
    subject = 'SEDAOriginatingAgencyIdentifier'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASubmissionAgencyIdentifier(EntityType):
    u""""""
    submission_agency_identifier = String(fulltextindexed=True)


class archive_transfer_submission_agency_identifier(RelationDefinition):
    name = 'seda_submission_agency_identifier'
    subject = 'SEDASubmissionAgencyIdentifier'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAStorageRule(EntityType):
    u""""""


class archive_transfer_storage_rule(RelationDefinition):
    name = 'seda_storage_rule'
    subject = 'SEDAStorageRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAppraisalRule(EntityType):
    u""""""


class archive_transfer_appraisal_rule(RelationDefinition):
    name = 'seda_appraisal_rule'
    subject = 'SEDAAppraisalRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAccessRule(EntityType):
    u""""""


class archive_transfer_access_rule(RelationDefinition):
    name = 'seda_access_rule'
    subject = 'SEDAAccessRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADisseminationRule(EntityType):
    u""""""


class archive_transfer_dissemination_rule(RelationDefinition):
    name = 'seda_dissemination_rule'
    subject = 'SEDADisseminationRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAReuseRule(EntityType):
    u""""""


class archive_transfer_reuse_rule(RelationDefinition):
    name = 'seda_reuse_rule'
    subject = 'SEDAReuseRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAClassificationRule(EntityType):
    u""""""
    classification_owner = String(fulltextindexed=True)


class archive_transfer_classification_rule(RelationDefinition):
    name = 'seda_classification_rule'
    subject = 'SEDAClassificationRule'
    object = 'SEDAArchiveTransfer'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDANeedAuthorization(EntityType):
    u""""""
    need_authorization = Boolean()


class archive_transfer_need_authorization(RelationDefinition):
    name = 'seda_need_authorization'
    subject = 'SEDANeedAuthorization'
    object = ('SEDAArchiveTransfer', 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltArchiveUnitArchiveUnitRefId(EntityType):
    u""""""


class archive_unit_alt_archive_unit_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_archive_unit_archive_unit_ref_id'
    subject = 'SEDAArchiveUnit'
    object = 'SEDAAltArchiveUnitArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

class relationship_type_relationship(RelationDefinition):
    name = 'seda_type_relationship'
    subject = 'SEDARelationship'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_relationship_code_list_version_from AT, CACLV seda_relationship_code_list_version_to CS,S container AT')]

class relationship_target(RelationDefinition):
    name = 'seda_target'
    subject = 'SEDARelationship'
    object = ('SEDABinaryDataObject', 'SEDAPhysicalDataObject')
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('S container C, O container C')]

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAttachment(EntityType):
    u""""""


class alt_binary_data_object_attachment_attachment(RelationDefinition):
    name = 'seda_attachment'
    subject = 'SEDAAttachment'
    object = 'SEDAAltBinaryDataObjectAttachment'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAUri(EntityType):
    u""""""


class alt_binary_data_object_attachment_uri(RelationDefinition):
    name = 'seda_uri'
    subject = 'SEDAUri'
    object = 'SEDAAltBinaryDataObjectAttachment'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class binary_data_object_algorithm(RelationDefinition):
    name = 'seda_algorithm'
    subject = 'SEDABinaryDataObject'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(S container AT, CACLV seda_message_digest_algorithm_code_list_version_from AT,        CACLV seda_message_digest_algorithm_code_list_version_to CS) OR EXISTS(S container AU, AU is SEDAArchiveUnit,           CS scheme_relation_type RT, RT name "seda_algorithm",           CS scheme_entity_type ET, ET name "SEDABinaryDataObject")')]

class compressed_algorithm(RelationDefinition):
    name = 'seda_algorithm'
    subject = 'SEDACompressed'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_compression_algorithm_code_list_version_from AT, CACLV seda_compression_algorithm_code_list_version_to CS,S container AT')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAFormatLitteral(EntityType):
    u""""""
    format_litteral = String(fulltextindexed=True)


class binary_data_object_format_litteral(RelationDefinition):
    name = 'seda_format_litteral'
    subject = 'SEDAFormatLitteral'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAMimeType(EntityType):
    u""""""


class binary_data_object_mime_type_from(RelationDefinition):
    name = 'seda_mime_type_from'
    subject = 'SEDAMimeType'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class mime_type_mime_type_to(RelationDefinition):
    name = 'seda_mime_type_to'
    subject = 'SEDAMimeType'
    object = 'Concept'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACLV seda_mime_type_code_list_version_from AT,        CACLV seda_mime_type_code_list_version_to CS,       S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT,            RT name "file_category")')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAFormatId(EntityType):
    u""""""


class binary_data_object_format_id_from(RelationDefinition):
    name = 'seda_format_id_from'
    subject = 'SEDAFormatId'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class format_id_format_id_to(RelationDefinition):
    name = 'seda_format_id_to'
    subject = 'SEDAFormatId'
    object = 'Concept'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACL seda_file_format_code_list_version_from AT,        CACL seda_file_format_code_list_version_to CS,        S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT,            RT name "file_category")')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEncoding(EntityType):
    u""""""


class binary_data_object_encoding_from(RelationDefinition):
    name = 'seda_encoding_from'
    subject = 'SEDAEncoding'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class encoding_encoding_to(RelationDefinition):
    name = 'seda_encoding_to'
    subject = 'SEDAEncoding'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACLV seda_encoding_code_list_version_from AT,        CACLV seda_encoding_code_list_version_to CS,       S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT,            RT name "seda_encoding_to")')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACreatingApplicationName(EntityType):
    u""""""
    creating_application_name = String(fulltextindexed=True)


class binary_data_object_creating_application_name(RelationDefinition):
    name = 'seda_creating_application_name'
    subject = 'SEDACreatingApplicationName'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACreatingApplicationVersion(EntityType):
    u""""""
    creating_application_version = String(fulltextindexed=True)


class binary_data_object_creating_application_version(RelationDefinition):
    name = 'seda_creating_application_version'
    subject = 'SEDACreatingApplicationVersion'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADateCreatedByApplication(EntityType):
    u""""""


class binary_data_object_date_created_by_application(RelationDefinition):
    name = 'seda_date_created_by_application'
    subject = 'SEDADateCreatedByApplication'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACreatingOs(EntityType):
    u""""""
    creating_os = String(fulltextindexed=True)


class binary_data_object_creating_os(RelationDefinition):
    name = 'seda_creating_os'
    subject = 'SEDACreatingOs'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACreatingOsVersion(EntityType):
    u""""""
    creating_os_version = String(fulltextindexed=True)


class binary_data_object_creating_os_version(RelationDefinition):
    name = 'seda_creating_os_version'
    subject = 'SEDACreatingOsVersion'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDALastModified(EntityType):
    u""""""


class binary_data_object_last_modified(RelationDefinition):
    name = 'seda_last_modified'
    subject = 'SEDALastModified'
    object = 'SEDABinaryDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAWidth(EntityType):
    u""""""


class physical_data_object_width(RelationDefinition):
    name = 'seda_width'
    subject = 'SEDAWidth'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAHeight(EntityType):
    u""""""


class physical_data_object_height(RelationDefinition):
    name = 'seda_height'
    subject = 'SEDAHeight'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADepth(EntityType):
    u""""""


class physical_data_object_depth(RelationDefinition):
    name = 'seda_depth'
    subject = 'SEDADepth'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAShape(EntityType):
    u""""""


class physical_data_object_shape(RelationDefinition):
    name = 'seda_shape'
    subject = 'SEDAShape'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADiameter(EntityType):
    u""""""


class physical_data_object_diameter(RelationDefinition):
    name = 'seda_diameter'
    subject = 'SEDADiameter'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDALength(EntityType):
    u""""""


class physical_data_object_length(RelationDefinition):
    name = 'seda_length'
    subject = 'SEDALength'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAThickness(EntityType):
    u""""""


class physical_data_object_thickness(RelationDefinition):
    name = 'seda_thickness'
    subject = 'SEDAThickness'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAWeight(EntityType):
    u""""""


class physical_data_object_weight(RelationDefinition):
    name = 'seda_weight'
    subject = 'SEDAWeight'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDANumberOfPage(EntityType):
    u""""""


class physical_data_object_number_of_page(RelationDefinition):
    name = 'seda_number_of_page'
    subject = 'SEDANumberOfPage'
    object = 'SEDAPhysicalDataObject'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAArchiveUnitRefId(EntityType):
    u""""""


class alt_archive_unit_archive_unit_ref_id_archive_unit_ref_id_from(RelationDefinition):
    name = 'seda_archive_unit_ref_id_from'
    subject = 'SEDAArchiveUnitRefId'
    object = ('SEDAAltArchiveUnitArchiveUnitRefId', 'SEDAAltIsPartOfArchiveUnitRefId', 'SEDAAltIsVersionOfArchiveUnitRefId', 'SEDAAltReferencesArchiveUnitRefId', 'SEDAAltReplacesArchiveUnitRefId', 'SEDAAltRequiresArchiveUnitRefId')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class archive_unit_ref_id_archive_unit_ref_id_to(RelationDefinition):
    name = 'seda_archive_unit_ref_id_to'
    subject = 'SEDAArchiveUnitRefId'
    object = 'SEDAArchiveUnit'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('S container C, O container C')]

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDASeqAltArchiveUnitArchiveUnitRefIdManagement(EntityType):
    u""""""


class alt_archive_unit_archive_unit_ref_id_seq_alt_archive_unit_archive_unit_ref_id_management(RelationDefinition):
    name = 'seda_seq_alt_archive_unit_archive_unit_ref_id_management'
    subject = 'SEDAAltArchiveUnitArchiveUnitRefId'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqStorageRuleRule(EntityType):
    u""""""


class storage_rule_seq_storage_rule_rule(RelationDefinition):
    name = 'seda_seq_storage_rule_rule'
    subject = 'SEDAStorageRule'
    object = 'SEDASeqStorageRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltStorageRulePreventInheritance(EntityType):
    u""""""


class storage_rule_alt_storage_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_storage_rule_prevent_inheritance'
    subject = 'SEDAStorageRule'
    object = 'SEDAAltStorageRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

class storage_rule_final_action(RelationDefinition):
    name = 'seda_final_action'
    subject = 'SEDAStorageRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_final_action", CS scheme_entity_type ET, ET name "SEDAStorageRule"')]

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqAppraisalRuleRule(EntityType):
    u""""""


class appraisal_rule_seq_appraisal_rule_rule(RelationDefinition):
    name = 'seda_seq_appraisal_rule_rule'
    subject = 'SEDAAppraisalRule'
    object = 'SEDASeqAppraisalRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltAppraisalRulePreventInheritance(EntityType):
    u""""""


class appraisal_rule_alt_appraisal_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_appraisal_rule_prevent_inheritance'
    subject = 'SEDAAppraisalRule'
    object = 'SEDAAltAppraisalRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

class appraisal_rule_final_action(RelationDefinition):
    name = 'seda_final_action'
    subject = 'SEDAAppraisalRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_final_action", CS scheme_entity_type ET, ET name "SEDAAppraisalRule"')]

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqAccessRuleRule(EntityType):
    u""""""


class access_rule_seq_access_rule_rule(RelationDefinition):
    name = 'seda_seq_access_rule_rule'
    subject = 'SEDAAccessRule'
    object = 'SEDASeqAccessRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltAccessRulePreventInheritance(EntityType):
    u""""""


class access_rule_alt_access_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_access_rule_prevent_inheritance'
    subject = 'SEDAAccessRule'
    object = 'SEDAAltAccessRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqDisseminationRuleRule(EntityType):
    u""""""


class dissemination_rule_seq_dissemination_rule_rule(RelationDefinition):
    name = 'seda_seq_dissemination_rule_rule'
    subject = 'SEDADisseminationRule'
    object = 'SEDASeqDisseminationRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltDisseminationRulePreventInheritance(EntityType):
    u""""""


class dissemination_rule_alt_dissemination_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_dissemination_rule_prevent_inheritance'
    subject = 'SEDADisseminationRule'
    object = 'SEDAAltDisseminationRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqReuseRuleRule(EntityType):
    u""""""


class reuse_rule_seq_reuse_rule_rule(RelationDefinition):
    name = 'seda_seq_reuse_rule_rule'
    subject = 'SEDAReuseRule'
    object = 'SEDASeqReuseRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltReuseRulePreventInheritance(EntityType):
    u""""""


class reuse_rule_alt_reuse_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_reuse_rule_prevent_inheritance'
    subject = 'SEDAReuseRule'
    object = 'SEDAAltReuseRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASeqClassificationRuleRule(EntityType):
    u""""""


class classification_rule_seq_classification_rule_rule(RelationDefinition):
    name = 'seda_seq_classification_rule_rule'
    subject = 'SEDAClassificationRule'
    object = 'SEDASeqClassificationRuleRule'
    cardinality = '*1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAltClassificationRulePreventInheritance(EntityType):
    u""""""


class classification_rule_alt_classification_rule_prevent_inheritance(RelationDefinition):
    name = 'seda_alt_classification_rule_prevent_inheritance'
    subject = 'SEDAClassificationRule'
    object = 'SEDAAltClassificationRulePreventInheritance'
    cardinality = '?1'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

class classification_rule_classification_level(RelationDefinition):
    name = 'seda_classification_level'
    subject = 'SEDAClassificationRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_classification_level"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAClassificationReassessingDate(EntityType):
    u""""""


class classification_rule_classification_reassessing_date(RelationDefinition):
    name = 'seda_classification_reassessing_date'
    subject = 'SEDAClassificationReassessingDate'
    object = 'SEDAClassificationRule'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDANeedReassessingAuthorization(EntityType):
    u""""""
    need_reassessing_authorization = Boolean()


class classification_rule_need_reassessing_authorization(RelationDefinition):
    name = 'seda_need_reassessing_authorization'
    subject = 'SEDANeedReassessingAuthorization'
    object = 'SEDAClassificationRule'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_archive_unit(RelationDefinition):
    name = 'seda_archive_unit'
    subject = 'SEDAArchiveUnit'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '?*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDADataObjectReference(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class width_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDAWidth'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDAWidth"')]

class height_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDAHeight'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDAHeight"')]

class depth_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDADepth'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDADepth"')]

class diameter_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDADiameter'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDADiameter"')]

class length_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDALength'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDALength"')]

class thickness_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDAThickness'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDAThickness"')]

class weight_unit(RelationDefinition):
    name = 'seda_unit'
    subject = 'SEDAWeight'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_unit", CS scheme_entity_type ET, ET name "SEDAWeight"')]

class seq_alt_archive_unit_archive_unit_ref_id_management_storage_rule(RelationDefinition):
    name = 'seda_storage_rule'
    subject = 'SEDAStorageRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_appraisal_rule(RelationDefinition):
    name = 'seda_appraisal_rule'
    subject = 'SEDAAppraisalRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_access_rule(RelationDefinition):
    name = 'seda_access_rule'
    subject = 'SEDAAccessRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_dissemination_rule(RelationDefinition):
    name = 'seda_dissemination_rule'
    subject = 'SEDADisseminationRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_reuse_rule(RelationDefinition):
    name = 'seda_reuse_rule'
    subject = 'SEDAReuseRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_classification_rule(RelationDefinition):
    name = 'seda_classification_rule'
    subject = 'SEDAClassificationRule'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class seq_alt_archive_unit_archive_unit_ref_id_management_description_level(RelationDefinition):
    name = 'seda_description_level'
    subject = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    object = 'Concept'
    cardinality = '1*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_description_level"')]

@seda_profile_element(cardinalities=['1..n', '1'], default_cardinality='1',
                      annotable=True)
class SEDATitle(EntityType):
    u""""""
    title = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_title(RelationDefinition):
    name = 'seda_title'
    subject = 'SEDATitle'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '11'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAFilePlanPosition(EntityType):
    u""""""
    file_plan_position = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_file_plan_position(RelationDefinition):
    name = 'seda_file_plan_position'
    subject = 'SEDAFilePlanPosition'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASystemId(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_system_id(RelationDefinition):
    name = 'seda_system_id'
    subject = 'SEDASystemId'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAOriginatingSystemId(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_originating_system_id(RelationDefinition):
    name = 'seda_originating_system_id'
    subject = 'SEDAOriginatingSystemId'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAArchivalAgencyArchiveUnitIdentifier(EntityType):
    u""""""
    archival_agency_archive_unit_identifier = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_archival_agency_archive_unit_identifier(RelationDefinition):
    name = 'seda_archival_agency_archive_unit_identifier'
    subject = 'SEDAArchivalAgencyArchiveUnitIdentifier'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAOriginatingAgencyArchiveUnitIdentifier(EntityType):
    u""""""
    originating_agency_archive_unit_identifier = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_originating_agency_archive_unit_identifier(RelationDefinition):
    name = 'seda_originating_agency_archive_unit_identifier'
    subject = 'SEDAOriginatingAgencyArchiveUnitIdentifier'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDATransferringAgencyArchiveUnitIdentifier(EntityType):
    u""""""
    transferring_agency_archive_unit_identifier = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_transferring_agency_archive_unit_identifier(RelationDefinition):
    name = 'seda_transferring_agency_archive_unit_identifier'
    subject = 'SEDATransferringAgencyArchiveUnitIdentifier'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDADescription(EntityType):
    u""""""
    description = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_description(RelationDefinition):
    name = 'seda_description'
    subject = 'SEDADescription'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAType(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_type_from(RelationDefinition):
    name = 'seda_type_from'
    subject = 'SEDAType'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class type_type_to(RelationDefinition):
    name = 'seda_type_to'
    subject = 'SEDAType'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_type_to"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADocumentType(EntityType):
    u""""""
    document_type = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_document_type(RelationDefinition):
    name = 'seda_document_type'
    subject = 'SEDADocumentType'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDALanguage(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_language_from(RelationDefinition):
    name = 'seda_language_from'
    subject = 'SEDALanguage'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class language_language_to(RelationDefinition):
    name = 'seda_language_to'
    subject = 'SEDALanguage'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_language_to"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDADescriptionLanguage(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_description_language_from(RelationDefinition):
    name = 'seda_description_language_from'
    subject = 'SEDADescriptionLanguage'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class description_language_description_language_to(RelationDefinition):
    name = 'seda_description_language_to'
    subject = 'SEDADescriptionLanguage'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_description_language_to"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAStatus(EntityType):
    u""""""
    status = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_status(RelationDefinition):
    name = 'seda_status'
    subject = 'SEDAStatus'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAVersion(EntityType):
    u""""""
    version = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_version(RelationDefinition):
    name = 'seda_version'
    subject = 'SEDAVersion'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDATag(EntityType):
    u""""""
    tag = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_tag(RelationDefinition):
    name = 'seda_tag'
    subject = 'SEDATag'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAKeyword(EntityType):
    u""""""
    keyword_content = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_keyword(RelationDefinition):
    name = 'seda_keyword'
    subject = 'SEDAKeyword'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAOriginatingAgency(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_originating_agency_from(RelationDefinition):
    name = 'seda_originating_agency_from'
    subject = 'SEDAOriginatingAgency'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class originating_agency_originating_agency_to(RelationDefinition):
    name = 'seda_originating_agency_to'
    subject = 'SEDAOriginatingAgency'
    object = 'AuthorityRecord'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASubmissionAgency(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_submission_agency_from(RelationDefinition):
    name = 'seda_submission_agency_from'
    subject = 'SEDASubmissionAgency'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class submission_agency_submission_agency_to(RelationDefinition):
    name = 'seda_submission_agency_to'
    subject = 'SEDASubmissionAgency'
    object = 'AuthorityRecord'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAuthorizedAgent(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_authorized_agent_from(RelationDefinition):
    name = 'seda_authorized_agent_from'
    subject = 'SEDAAuthorizedAgent'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class authorized_agent_authorized_agent_to(RelationDefinition):
    name = 'seda_authorized_agent_to'
    subject = 'SEDAAuthorizedAgent'
    object = 'AuthorityRecord'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAWriter(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_writer_from(RelationDefinition):
    name = 'seda_writer_from'
    subject = 'SEDAWriter'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class writer_writer_to(RelationDefinition):
    name = 'seda_writer_to'
    subject = 'SEDAWriter'
    object = 'AuthorityRecord'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAAddressee(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_addressee_from(RelationDefinition):
    name = 'seda_addressee_from'
    subject = 'SEDAAddressee'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class addressee_addressee_to(RelationDefinition):
    name = 'seda_addressee_to'
    subject = 'SEDAAddressee'
    object = 'AuthorityRecord'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDARecipient(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_recipient_from(RelationDefinition):
    name = 'seda_recipient_from'
    subject = 'SEDARecipient'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class recipient_recipient_to(RelationDefinition):
    name = 'seda_recipient_to'
    subject = 'SEDARecipient'
    object = 'AuthorityRecord'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASource(EntityType):
    u""""""
    source = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_source(RelationDefinition):
    name = 'seda_source'
    subject = 'SEDASource'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACreatedDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_created_date(RelationDefinition):
    name = 'seda_created_date'
    subject = 'SEDACreatedDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDATransactedDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_transacted_date(RelationDefinition):
    name = 'seda_transacted_date'
    subject = 'SEDATransactedDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAAcquiredDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_acquired_date(RelationDefinition):
    name = 'seda_acquired_date'
    subject = 'SEDAAcquiredDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDASentDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_sent_date(RelationDefinition):
    name = 'seda_sent_date'
    subject = 'SEDASentDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAReceivedDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_received_date(RelationDefinition):
    name = 'seda_received_date'
    subject = 'SEDAReceivedDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDARegisteredDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_registered_date(RelationDefinition):
    name = 'seda_registered_date'
    subject = 'SEDARegisteredDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAStartDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_start_date(RelationDefinition):
    name = 'seda_start_date'
    subject = 'SEDAStartDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEndDate(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_end_date(RelationDefinition):
    name = 'seda_end_date'
    subject = 'SEDAEndDate'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAEvent(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_event(RelationDefinition):
    name = 'seda_event'
    subject = 'SEDAEvent'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class data_object_reference_data_object_reference_id(RelationDefinition):
    name = 'seda_data_object_reference_id'
    subject = 'SEDADataObjectReference'
    object = ('SEDABinaryDataObject', 'SEDAPhysicalDataObject')
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('S container C, O container C')]

class seq_storage_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqStorageRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_storage_rule_code_list_version_from AT, CACLV seda_storage_rule_code_list_version_to CS,S container AT')]

class seq_storage_rule_rule_start_date(RelationDefinition):
    name = 'seda_start_date'
    subject = 'SEDAStartDate'
    object = ('SEDASeqAccessRuleRule', 'SEDASeqAppraisalRuleRule', 'SEDASeqClassificationRuleRule', 'SEDASeqDisseminationRuleRule', 'SEDASeqReuseRuleRule', 'SEDASeqStorageRuleRule')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAPreventInheritance(EntityType):
    u""""""
    prevent_inheritance = Boolean()


class alt_storage_rule_prevent_inheritance_prevent_inheritance(RelationDefinition):
    name = 'seda_prevent_inheritance'
    subject = 'SEDAPreventInheritance'
    object = ('SEDAAltAccessRulePreventInheritance', 'SEDAAltAppraisalRulePreventInheritance', 'SEDAAltClassificationRulePreventInheritance', 'SEDAAltDisseminationRulePreventInheritance', 'SEDAAltReuseRulePreventInheritance', 'SEDAAltStorageRulePreventInheritance')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1..n', '1'], default_cardinality='1',
                      annotable=True)
class SEDARefNonRuleId(EntityType):
    u""""""


class alt_storage_rule_prevent_inheritance_ref_non_rule_id_from(RelationDefinition):
    name = 'seda_ref_non_rule_id_from'
    subject = 'SEDARefNonRuleId'
    object = ('SEDAAltAccessRulePreventInheritance', 'SEDAAltAppraisalRulePreventInheritance', 'SEDAAltClassificationRulePreventInheritance', 'SEDAAltDisseminationRulePreventInheritance', 'SEDAAltReuseRulePreventInheritance', 'SEDAAltStorageRulePreventInheritance')
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class ref_non_rule_id_ref_non_rule_id_to(RelationDefinition):
    name = 'seda_ref_non_rule_id_to'
    subject = 'SEDARefNonRuleId'
    object = 'Concept'
    cardinality = '**'
    composite = fulltext_container = None
    inlined = False
    constraints = []

class seq_appraisal_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqAppraisalRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACLV seda_appraisal_rule_code_list_version_from AT,        CACLV seda_appraisal_rule_code_list_version_to CS,       S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit)')]

class seq_access_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqAccessRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, EXISTS(CACLV seda_access_rule_code_list_version_from AT,        CACLV seda_access_rule_code_list_version_to CS,       S container AT) OR EXISTS(S container AU, AU is SEDAArchiveUnit)')]

class seq_dissemination_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqDisseminationRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_dissemination_rule_code_list_version_from AT, CACLV seda_dissemination_rule_code_list_version_to CS,S container AT')]

class seq_reuse_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqReuseRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_reuse_rule_code_list_version_from AT, CACLV seda_reuse_rule_code_list_version_to CS,S container AT')]

class seq_classification_rule_rule_rule(RelationDefinition):
    name = 'seda_rule'
    subject = 'SEDASeqClassificationRuleRule'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CACLV seda_classification_rule_code_list_version_from AT, CACLV seda_classification_rule_code_list_version_to CS,S container AT')]

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDACustodialHistoryItem(EntityType):
    u""""""
    custodial_history_item = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_custodial_history_item(RelationDefinition):
    name = 'seda_custodial_history_item'
    subject = 'SEDACustodialHistoryItem'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDACustodialHistoryFile(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_custodial_history_file(RelationDefinition):
    name = 'seda_custodial_history_file'
    subject = 'SEDACustodialHistoryFile'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAKeywordReference(EntityType):
    u""""""


class keyword_keyword_reference_from(RelationDefinition):
    name = 'seda_keyword_reference_from'
    subject = 'SEDAKeywordReference'
    object = 'SEDAKeyword'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class keyword_reference_keyword_reference_to(RelationDefinition):
    name = 'seda_keyword_reference_to'
    subject = 'SEDAKeywordReference'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, S seda_keyword_reference_to_scheme CS')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAKeywordType(EntityType):
    u""""""


class keyword_keyword_type_from(RelationDefinition):
    name = 'seda_keyword_type_from'
    subject = 'SEDAKeywordType'
    object = 'SEDAKeyword'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class keyword_type_keyword_type_to(RelationDefinition):
    name = 'seda_keyword_type_to'
    subject = 'SEDAKeywordType'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_keyword_type_to"')]

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDASpatial(EntityType):
    u""""""
    spatial = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_spatial(RelationDefinition):
    name = 'seda_spatial'
    subject = 'SEDASpatial'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDATemporal(EntityType):
    u""""""
    temporal = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_temporal(RelationDefinition):
    name = 'seda_temporal'
    subject = 'SEDATemporal'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAJuridictional(EntityType):
    u""""""
    juridictional = String(fulltextindexed=True)


class seq_alt_archive_unit_archive_unit_ref_id_management_juridictional(RelationDefinition):
    name = 'seda_juridictional'
    subject = 'SEDAJuridictional'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAIsVersionOf(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_is_version_of(RelationDefinition):
    name = 'seda_is_version_of'
    subject = 'SEDAIsVersionOf'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAReplaces(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_replaces(RelationDefinition):
    name = 'seda_replaces'
    subject = 'SEDAReplaces'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDARequires(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_requires(RelationDefinition):
    name = 'seda_requires'
    subject = 'SEDARequires'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAIsPartOf(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_is_part_of(RelationDefinition):
    name = 'seda_is_part_of'
    subject = 'SEDAIsPartOf'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '0..n', '1', '1..n'], default_cardinality='1',
                      annotable=True)
class SEDAReferences(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_references(RelationDefinition):
    name = 'seda_references'
    subject = 'SEDAReferences'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1*'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEventIdentifier(EntityType):
    u""""""


class event_event_identifier(RelationDefinition):
    name = 'seda_event_identifier'
    subject = 'SEDAEventIdentifier'
    object = 'SEDAEvent'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEventType(EntityType):
    u""""""


class event_event_type_from(RelationDefinition):
    name = 'seda_event_type_from'
    subject = 'SEDAEventType'
    object = 'SEDAEvent'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class event_type_event_type_to(RelationDefinition):
    name = 'seda_event_type_to'
    subject = 'SEDAEventType'
    object = 'Concept'
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('O in_scheme CS, CS scheme_relation_type CR, CR name "seda_event_type_to"')]

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAEventDetail(EntityType):
    u""""""


class event_event_detail(RelationDefinition):
    name = 'seda_event_detail'
    subject = 'SEDAEventDetail'
    object = 'SEDAEvent'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsVersionID(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_version_id(RelationDefinition):
    name = 'seda_gps_version_id'
    subject = 'SEDAGpsVersionID'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsAltitude(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_altitude(RelationDefinition):
    name = 'seda_gps_altitude'
    subject = 'SEDAGpsAltitude'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsAltitudeRef(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_altitude_ref(RelationDefinition):
    name = 'seda_gps_altitude_ref'
    subject = 'SEDAGpsAltitudeRef'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsLatitude(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_latitude(RelationDefinition):
    name = 'seda_gps_latitude'
    subject = 'SEDAGpsLatitude'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsLatitudeRef(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_latitude_ref(RelationDefinition):
    name = 'seda_gps_latitude_ref'
    subject = 'SEDAGpsLatitudeRef'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsLongitude(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_longitude(RelationDefinition):
    name = 'seda_gps_longitude'
    subject = 'SEDAGpsLongitude'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsLongitudeRef(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_longitude_ref(RelationDefinition):
    name = 'seda_gps_longitude_ref'
    subject = 'SEDAGpsLongitudeRef'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAGpsDateStamp(EntityType):
    u""""""


class seq_alt_archive_unit_archive_unit_ref_id_management_gps_date_stamp(RelationDefinition):
    name = 'seda_gps_date_stamp'
    subject = 'SEDAGpsDateStamp'
    object = 'SEDASeqAltArchiveUnitArchiveUnitRefIdManagement'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['0..1', '1'], default_cardinality='1',
                      annotable=True)
class SEDAwhen(EntityType):
    u""""""


class custodial_history_item_when(RelationDefinition):
    name = 'seda_when'
    subject = 'SEDAwhen'
    object = 'SEDACustodialHistoryItem'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class custodial_history_file_data_object_reference_id(RelationDefinition):
    name = 'seda_data_object_reference_id'
    subject = 'SEDACustodialHistoryFile'
    object = ('SEDABinaryDataObject', 'SEDAPhysicalDataObject')
    cardinality = '?*'
    composite = fulltext_container = None
    inlined = True
    constraints = [RQLConstraint('S container C, O container C')]

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltIsVersionOfArchiveUnitRefId(EntityType):
    u""""""


class is_version_of_alt_is_version_of_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_is_version_of_archive_unit_ref_id'
    subject = 'SEDAIsVersionOf'
    object = 'SEDAAltIsVersionOfArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltReplacesArchiveUnitRefId(EntityType):
    u""""""


class replaces_alt_replaces_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_replaces_archive_unit_ref_id'
    subject = 'SEDAReplaces'
    object = 'SEDAAltReplacesArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltRequiresArchiveUnitRefId(EntityType):
    u""""""


class requires_alt_requires_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_requires_archive_unit_ref_id'
    subject = 'SEDARequires'
    object = 'SEDAAltRequiresArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltIsPartOfArchiveUnitRefId(EntityType):
    u""""""


class is_part_of_alt_is_part_of_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_is_part_of_archive_unit_ref_id'
    subject = 'SEDAIsPartOf'
    object = 'SEDAAltIsPartOfArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDAAltReferencesArchiveUnitRefId(EntityType):
    u""""""


class references_alt_references_archive_unit_ref_id(RelationDefinition):
    name = 'seda_alt_references_archive_unit_ref_id'
    subject = 'SEDAReferences'
    object = 'SEDAAltReferencesArchiveUnitRefId'
    cardinality = '11'
    composite = fulltext_container = 'subject'
    inlined = False
    constraints = []

class alt_is_version_of_archive_unit_ref_id_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDAAltIsVersionOfArchiveUnitRefId'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDARepositoryArchiveUnitPID(EntityType):
    u""""""


class alt_is_version_of_archive_unit_ref_id_repository_archive_unit_pid(RelationDefinition):
    name = 'seda_repository_archive_unit_pid'
    subject = 'SEDARepositoryArchiveUnitPID'
    object = ('SEDAAltIsPartOfArchiveUnitRefId', 'SEDAAltIsVersionOfArchiveUnitRefId', 'SEDAAltReferencesArchiveUnitRefId', 'SEDAAltReplacesArchiveUnitRefId', 'SEDAAltRequiresArchiveUnitRefId')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

@seda_profile_element(cardinalities=['1'], default_cardinality='1',
                      annotable=False)
class SEDARepositoryObjectPID(EntityType):
    u""""""


class alt_is_version_of_archive_unit_ref_id_repository_object_pid(RelationDefinition):
    name = 'seda_repository_object_pid'
    subject = 'SEDARepositoryObjectPID'
    object = ('SEDAAltIsPartOfArchiveUnitRefId', 'SEDAAltIsVersionOfArchiveUnitRefId', 'SEDAAltReferencesArchiveUnitRefId', 'SEDAAltReplacesArchiveUnitRefId', 'SEDAAltRequiresArchiveUnitRefId')
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class alt_replaces_archive_unit_ref_id_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDAAltReplacesArchiveUnitRefId'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class alt_requires_archive_unit_ref_id_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDAAltRequiresArchiveUnitRefId'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class alt_is_part_of_archive_unit_ref_id_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDAAltIsPartOfArchiveUnitRefId'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []

class alt_references_archive_unit_ref_id_data_object_reference(RelationDefinition):
    name = 'seda_data_object_reference'
    subject = 'SEDADataObjectReference'
    object = 'SEDAAltReferencesArchiveUnitRefId'
    cardinality = '1?'
    composite = fulltext_container = 'object'
    inlined = True
    constraints = []




# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-seda unit tests for hooks"""

from os.path import dirname, join
from itertools import chain, repeat

from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_compound.entities import copy_entity
from cubicweb_seda import testutils, dataimport


class ValidationHooksTC(CubicWebTC):

    assertValidationError = testutils.assertValidationError

    def test_ref_non_rule_constraints(self):
        with self.admin_access.cnx() as cnx:
            create = cnx.create_entity

            access_scheme = create('ConceptScheme', title=u'access')
            access_concept = access_scheme.add_concept(label=u'anyone')
            reuse_scheme = create('ConceptScheme', title=u'reuse')
            reuse_concept = reuse_scheme.add_concept(label=u'share-alike')
            cnx.commit()

            bdo = testutils.create_transfer_to_bdo(cnx)
            transfer = bdo.container[0]
            create('SEDAAccessRuleCodeListVersion',
                   seda_access_rule_code_list_version_from=transfer,
                   seda_access_rule_code_list_version_to=access_scheme)
            create('SEDAReuseRuleCodeListVersion',
                   seda_reuse_rule_code_list_version_from=transfer,
                   seda_reuse_rule_code_list_version_to=reuse_scheme)
            cnx.commit()

            rule_base = create('SEDAAccessRule', seda_access_rule=transfer)
            rule_alt = create('SEDAAltAccessRulePreventInheritance',
                              reverse_seda_alt_access_rule_prevent_inheritance=rule_base)
            non_rule = create('SEDARefNonRuleId', seda_ref_non_rule_id_from=rule_alt)
            cnx.commit()

            with self.assertValidationError(cnx) as cm:
                non_rule.cw_set(seda_ref_non_rule_id_to=reuse_concept)
            self.assertIn('seda_ref_non_rule_id_to-subject', cm.exception.errors)

            non_rule.cw_set(seda_ref_non_rule_id_to=access_concept)
            cnx.commit()

    def test_empty_choice_created(self):
        """Check that a ValidationError is raised when an empty SEDAAlt... entity is created."""
        with self.admin_access.cnx() as cnx:
            # Create an empty SEDAAltAccessRulePreventInheritance
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            access_rule = cnx.create_entity('SEDAAccessRule', seda_access_rule=transfer)
            with self.assertValidationError(cnx) as cm:
                cnx.create_entity('SEDAAltAccessRulePreventInheritance',
                                  reverse_seda_alt_access_rule_prevent_inheritance=access_rule)
            self.assertIn('An alternative cannot be empty',
                          str(cm.exception))

    def test_valid_choice_created(self):
        """Check that everything goes fine when a valid SEDAAlt... entity is created."""
        with self.admin_access.cnx() as cnx:
            # Create an empty SEDAArchiveUnit
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            access_rule = cnx.create_entity('SEDAAccessRule', seda_access_rule=transfer)
            choice = cnx.create_entity('SEDAAltAccessRulePreventInheritance',
                                       reverse_seda_alt_access_rule_prevent_inheritance=access_rule)
            cnx.create_entity('SEDAPreventInheritance', prevent_inheritance=False,
                              seda_prevent_inheritance=choice)
            cnx.commit()

    def test_last_item_in_choice_deleted(self):
        """Check that a ValidationError is raised when the last relation from a SEDAAlt... entity
        is deleted.
        """
        with self.admin_access.cnx() as cnx:
            # Create a valid SEDAAltAccessRulePreventInheritance
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            access_rule = cnx.create_entity('SEDAAccessRule', seda_access_rule=transfer)
            choice = cnx.create_entity('SEDAAltAccessRulePreventInheritance',
                                       reverse_seda_alt_access_rule_prevent_inheritance=access_rule)
            rel = cnx.create_entity('SEDAPreventInheritance', prevent_inheritance=False,
                                    seda_prevent_inheritance=choice)
            cnx.commit()
            # Delete SEDAPreventInheritance
            with self.assertValidationError(cnx) as cm:
                cnx.execute('DELETE SEDAPreventInheritance X WHERE X eid %(rel_eid)s',
                            {'rel_eid': rel.eid})
            self.assertIn('An alternative cannot be empty',
                          str(cm.exception))

    def test_item_in_choice_deleted_with_remaining_item(self):
        """Check that everything goes fine when the a relation from a SEDAAlt... entity
        is deleted but another relation remains.
        """
        with self.admin_access.cnx() as cnx:
            # Create a valid SEDAAltAccessRulePreventInheritance with two items
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            access_rule = cnx.create_entity('SEDAAccessRule', seda_access_rule=transfer)
            choice = cnx.create_entity('SEDAAltAccessRulePreventInheritance',
                                       reverse_seda_alt_access_rule_prevent_inheritance=access_rule)
            rel = cnx.create_entity('SEDAPreventInheritance', prevent_inheritance=False,
                                    seda_prevent_inheritance=choice)
            cnx.create_entity('SEDARefNonRuleId', seda_ref_non_rule_id_from=choice)
            cnx.commit()
            # Delete SEDAPreventInheritance
            cnx.execute('DELETE SEDAPreventInheritance X WHERE X eid %(rel_eid)s',
                        {'rel_eid': rel.eid})
            cnx.commit()


class SetDefaultHooksTC(CubicWebTC):

    def test_default_code_list_version(self):
        with self.admin_access.cnx() as cnx:
            for rtype, etype in chain(zip(('file_category', 'seda_encoding_to'),
                                          repeat(None)),
                                      [('seda_algorithm', 'SEDABinaryDataObject'),
                                       ('seda_rule', 'SEDASeqAppraisalRuleRule'),
                                       ('seda_rule', 'SEDASeqAccessRuleRule'),
                                       ('seda_rule', 'SEDASeqDisseminationRuleRule')]):
                testutils.scheme_for_type(cnx, rtype, etype)
            cnx.commit()
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            cnx.commit()

            self.assertTrue(transfer.reverse_seda_file_format_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_message_digest_algorithm_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_mime_type_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_encoding_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_access_rule_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_appraisal_rule_code_list_version_from)
            self.assertTrue(transfer.reverse_seda_dissemination_rule_code_list_version_from)

    def test_default_card_on_typed_data_object_ref(self):
        """When creating a SEDADataObjectReference in the context of a reference, its cardinality
        should always be 1 (by default any card is allowed since in may be used in the context of
        'main' reference).
        """
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            version_of = cnx.create_entity('SEDAIsVersionOf', seda_is_version_of=unit_alt_seq)
            alt2 = cnx.create_entity('SEDAAltIsVersionOfArchiveUnitRefId',
                                     reverse_seda_alt_is_version_of_archive_unit_ref_id=version_of)
            do_ref = cnx.create_entity('SEDADataObjectReference', seda_data_object_reference=alt2)
            cnx.commit()

            self.assertEqual(do_ref.user_cardinality, '1')

    def test_simplified_profile_do_ref_sync(self):
        """When creating a SEDABinaryDataObject in a simplified profile from the ui, one only
        specifies user_cardinality for it, not for it's associated reference whose cardinality
        should have the same value.
        """
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile',
                                         simplified_profile=True)
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            bdo = testutils.create_data_object(unit_alt_seq, user_cardinality=u'1',
                                               seda_binary_data_object=transfer)
            cnx.commit()
            ref = cnx.find('SEDADataObjectReference').one()

            ref.cw_clear_all_caches()
            self.assertEqual(ref.user_cardinality, '1')

            bdo.cw_set(user_cardinality=u'1..n')
            cnx.commit()
            ref.cw_clear_all_caches()
            self.assertEqual(ref.user_cardinality, '1..n')


class CheckProfileTC(CubicWebTC):

    assertValidationError = testutils.assertValidationError

    def test_base(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'diagnosis testing')
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            access_rule = cnx.create_entity('SEDAAccessRule', seda_access_rule=unit_alt_seq)
            cnx.commit()

            with self.assertValidationError(cnx):
                transfer.cw_set(simplified_profile=True)

            access_rule_seq = cnx.create_entity('SEDASeqAccessRuleRule',
                                                reverse_seda_seq_access_rule_rule=access_rule)
            start_date = cnx.create_entity('SEDAStartDate',
                                           seda_start_date=access_rule_seq)
            transfer.cw_set(simplified_profile=True)
            cnx.commit()

            with self.assertValidationError(cnx):
                start_date.cw_set(user_cardinality=u'0..1')

            with self.assertValidationError(cnx):
                access_rule_seq.cw_delete()


class DispatchFileCategoryTC(CubicWebTC):
    def setup_database(self):
        with self.admin_access.cnx() as cnx:
            dataimport.import_seda_schemes(cnx, lcsv_files=[
                (u'Categories de fichier',
                 'file_category', (),
                 join(dirname(__file__), 'data', 'file_categories.csv'))])
            self.categories = dict(cnx.execute(
                'Any LL, C WHERE L label_of C, L label LL, '
                'C in_scheme CS, CS title "Categories de fichier"'))
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile',
                                         simplified_profile=True)
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            bdo = testutils.create_data_object(unit_alt_seq,
                                               seda_binary_data_object=transfer)
            cnx.commit()

        self.transfer_eid = transfer.eid
        self.unit_alt_seq_eid = unit_alt_seq.eid
        self.bdo_eid = bdo.eid

    def test_base(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.entity_from_eid(self.transfer_eid)
            unit_alt_seq = cnx.entity_from_eid(self.unit_alt_seq_eid)
            bdo = cnx.entity_from_eid(self.bdo_eid)

            bdo.cw_set(file_category=self.categories['document'])
            cnx.commit()
            self.assertFormatEqual(bdo,
                                   ['application/msword', 'application/pdf'],
                                   ['fmt/37', 'fmt/38', 'fmt/14'])

            bdo.cw_set(file_category=None)
            cnx.commit()
            self.assertFormatEqual(bdo, [], [])

            bdo.cw_set(file_category=self.categories['doc'])
            cnx.commit()
            self.assertFormatEqual(bdo,
                                   ['application/msword'],
                                   ['fmt/37', 'fmt/38'])

            # test no dispatch occurs on component archive unit...
            unit_comp, unit_comp_alt, unit_comp_alt_seq = testutils.create_archive_unit(
                None, cnx=cnx)
            bdo_comp = testutils.create_data_object(unit_comp_alt_seq)
            cnx.commit()

            bdo_comp.cw_set(file_category=self.categories['document'])
            cnx.commit()
            self.assertFormatEqual(bdo_comp, [], [])
            # until it's imported into some parent
            copy_entity(unit_comp, seda_archive_unit=unit_alt_seq, clone_of=unit_comp)
            cnx.commit()
            imported_bdo, = [x for x in transfer.binary_data_objects if x.eid != bdo.eid]
            self.assertFormatEqual(imported_bdo,
                                   ['application/msword', 'application/pdf'],
                                   ['fmt/37', 'fmt/38', 'fmt/14'])

            # ensure deleting the data object doesn't raise an exception because
            # of erroneous hook
            bdo.cw_delete()
            cnx.commit()

    def test_join(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.entity_from_eid(self.transfer_eid)
            bdo = cnx.entity_from_eid(self.bdo_eid)

            formats = testutils.scheme_for_type(cnx, 'seda_format_id_to', None,
                                                u'fmt/37', u'fmt/14')
            mime_types = testutils.scheme_for_type(cnx, 'seda_mime_type_to', None,
                                                   u'application/msword', u'application/pdf')
            transfer.reverse_seda_file_format_code_list_version_from[0].cw_set(
                seda_file_format_code_list_version_to=formats)
            transfer.reverse_seda_mime_type_code_list_version_from[0].cw_set(
                seda_mime_type_code_list_version_to=mime_types)
            cnx.commit()

            bdo.cw_set(file_category=self.categories['document'])
            cnx.commit()
            self.assertFormatEqual(bdo,
                                   ['application/msword', 'application/pdf'],
                                   ['fmt/37', 'fmt/14'])

    def assertFormatEqual(self, bdo, expected_mime_types, expected_format_ids):
        bdo.cw_clear_all_caches()
        bdo.reverse_seda_mime_type_from[0].cw_clear_all_caches()
        bdo.reverse_seda_format_id_from[0].cw_clear_all_caches()
        mime_types = set(x.label() for x in bdo.reverse_seda_mime_type_from[0].seda_mime_type_to)
        self.assertEqual(mime_types, set(expected_mime_types))
        format_ids = set(x.label() for x in bdo.reverse_seda_format_id_from[0].seda_format_id_to)
        self.assertEqual(format_ids, set(expected_format_ids))


class OrderingHooksTC(CubicWebTC):

    def test_first_level_archive_units(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            unit1 = testutils.create_archive_unit(transfer)[0]
            unit2 = testutils.create_archive_unit(transfer)[0]
            cnx.commit()

            self.assertEqual(unit1.ordering, 1)
            self.assertEqual(unit2.ordering, 2)

    def test_archive_units(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            unit1 = testutils.create_archive_unit(unit_alt_seq)[0]
            unit2 = testutils.create_archive_unit(unit_alt_seq)[0]
            cnx.commit()

            self.assertEqual(unit1.ordering, 1)
            self.assertEqual(unit2.ordering, 2)

    def test_data_objects(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile',
                                         simplified_profile=True)
            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
            bdo1 = testutils.create_data_object(unit_alt_seq, user_cardinality=u'1',
                                                seda_binary_data_object=transfer)
            bdo2 = testutils.create_data_object(unit_alt_seq, user_cardinality=u'1',
                                                seda_binary_data_object=transfer)
            cnx.commit()

            self.assertEqual(bdo1.ordering, 1)
            self.assertEqual(bdo2.ordering, 2)
            self.assertEqual(bdo1.reverse_seda_data_object_reference_id[0].ordering, 1)
            self.assertEqual(bdo2.reverse_seda_data_object_reference_id[0].ordering, 2)

    def test_remove_keep_in_sync(self):
        with self.admin_access.cnx() as cnx:
            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
            bdo1 = testutils.create_data_object(transfer)
            bdo2 = testutils.create_data_object(transfer)
            cnx.commit()

            bdo1.cw_delete()
            cnx.commit()

            bdo2.cw_clear_all_caches()
            self.assertEqual(bdo2.ordering, 1)


if __name__ == '__main__':
    import unittest
    unittest.main()
